@@ -1,5 +1,5 @@
 import re
-from railgun._helper import HybridObj, iteralt, product
+from railgun._helper import iteralt, product
 
 CJOINSTR = '_'
 
@@ -7,7 +7,7 @@ RE_CFDEC_FUNC = re.compile(
     r"(?:(?P<ret>[\w]+) )?(?P<fname>[^\( ][^\(]*)\((?P<args>[^\)]*)\)"
     )  # matches "ret func(a, b, c)" as "RET FNAME(ARGS)"
 RE_CFDEC_CHOSET = re.compile(
-    r"{(?P<key>[a-zA-Z]\w*) *\| *(?P<choices>[\w, ]+)}",
+    r"{ *(?P<key>[a-zA-Z]\w*) *\| *(?P<choices>[\w, ]+)}",
     )  # matches "{key | a, b, c}" as "{KEY | CHOICES}"
 RE_CFDEC_CHOSET_SPLIT = re.compile(
     r"{[a-zA-Z]\w* *\| *[\w, ]+}",
@@ -18,6 +18,54 @@ RE_CFDEC_ARG = re.compile(
     )  # matches "int a=1" as "CDT NAME=DEFAULT"
 
 
+class _CFunctionDeclaration(object):
+    """
+    A class to sotre parsed information of C-function declaration
+
+    Attributes
+    ----------
+    ret : str or None
+        returned value
+    fname : str
+        name of function
+    choset : list of {'choices': [str], 'key': str}
+        choice set
+    args : list of {'aname': [str], 'default': obj, 'cdt': str}
+        arguments
+    fnget : (str, str, ...) -> str
+        function name getter.
+        it constructs C function name from given "choices".
+
+    """
+
+    def as_dict(self):
+        keys = ["ret", "fname", "choset", "args", "fnget"]
+        return dict((k, getattr(self, k)) for k in keys if hasattr(self, k))
+
+    @classmethod
+    def from_string(cls, cfstr):
+        cfstr = cfstr.strip()  # "'ans' func_{k|a,b,c}(int a, i i=1)"
+
+        match_func = RE_CFDEC_FUNC.match(cfstr)
+        # ret="ans", fname="func_{k|a,b,c}", args="int a, i i=1"
+        if match_func:
+            return cls.from_groupdict(**match_func.groupdict())
+        else:
+            raise ValueError(
+                "%s is invalid as c-function cdt declaration" % cfstr)
+
+    @classmethod
+    def from_groupdict(cls, ret, fname, args):
+        parsed = cls()
+        (fname, choset, fnget) = cfdec_fname_parse(fname)
+        parsed.ret = ret
+        parsed.fname = fname
+        parsed.choset = choset
+        parsed.args = cfdec_args_parse(args)
+        parsed.fnget = fnget
+        return parsed
+
+
 def cfdec_args_parse(args):
     """Parse cfdec argument like this: 'int a=1, float x=2.0'"""
     parsed = []
@@ -54,43 +102,8 @@ def cfdec_fname_parse(rawfname):
     return (fname, choset, fnget)
 
 
-def cfdec_parse(cfstr):
-    """
-    Parse declaration of a function
-
-    Given declaration of a function,::
-
-        ret func_{k1|x,y,z}_...(int a, ...)
-
-    this function returns dict like this::
-
-        {'args': [{'aname': 'a', 'default': None, 'cdt': 'int'},
-                  ...],
-         'choset': [{'choices': ['x', 'y', 'z'], 'key': 'k1'},
-                     ...],
-         'fname': 'func',
-         'fnget': <function fnget_func at XXXXXXXXX>
-         'ret': 'ret'}
-
-    """
-    cfstr = cfstr.strip()  # "'ans' func_{k|a,b,c}(int a, i i=1)"
-
-    match_func = RE_CFDEC_FUNC.match(cfstr)
-    if match_func:
-        parsed = HybridObj(match_func.groupdict())
-        ## ret="ans", fname="func_{k|a,b,c}", args="int a, i i=1"
-    else:
-        raise ValueError(
-            "%s is invalid as c-function cdt declaration" % cfstr)
-
-    (fname, choset, fnget) = cfdec_fname_parse(parsed.fname)
-    parsed.fname = fname
-    parsed.choset = choset
-    parsed.args = cfdec_args_parse(parsed.args)
-    parsed.fnget = fnget
-
-    return parsed
+cfdec_parse = _CFunctionDeclaration.from_string
 
 
 def choice_combinations(cfdec):
-    return product([c['choices'] for c in cfdec['choset']])
+    return product([c['choices'] for c in cfdec.choset])
