@@ -268,8 +268,11 @@ def gene_cfpywrap(attrs, cfdec):
             else:
                 return
         else:
-            raise RuntimeError('c-function %s() terminates with code %d'
-                               % (cfname, rcode))
+            if rcode in self._cerrors_:
+                raise self._cerrors_[rcode]
+            else:
+                raise RuntimeError('c-function %s() terminates with code %d'
+                                   % (cfname, rcode))
     cfpywrap.func_name = cfdec.fname
     # wrap it if there is wrap function
     wrap_name = '_cwrap_%s' % cfdec.fname
@@ -377,7 +380,10 @@ def get_struct_class(cmems_parsed_list, cstructname):
         if parsed.valtype == 'object':
             fields.append((parsed.vname, parsed.cdt._ctype_))
         else:
-            ctype = POINTER_nth(CDT2CTYPE[parsed.cdt], parsed.ndim)
+            if parsed.carrtype == "flat":
+                ctype = POINTER(CDT2CTYPE[parsed.cdt])
+            else:
+                ctype = POINTER_nth(CDT2CTYPE[parsed.cdt], parsed.ndim)
             fields.append((parsed.vname, ctype))
 
     class StructClass(Structure):
@@ -410,7 +416,7 @@ def load_cfunc(cdll, cfuncs_parsed, struct_type_p, cfuncprefix, idxset):
             cf = cdll[cfuncprefix + cfname]
             cf.restype = c_int
             cf.argtypes = (
-                [struct_type_p] + map(get_arg_ct, parsed['args']))
+                [struct_type_p] + map(get_arg_ct, parsed.args))
             cfunc_loaded[cfname] = cf
     return cfunc_loaded
 
@@ -539,6 +545,7 @@ class MetaSimObject(type):
 class SimObject(object):
     __metaclass__ = MetaSimObject
     _calloc_ = True  # if True, use cstyle.CStyle to allocate memory
+    _cerrors_ = {}
 
     def __init__(self, **kwds):
         """
@@ -569,6 +576,28 @@ class SimObject(object):
 
         self._set_all(**kwds)
 
+    def __setstate__(self, state):
+        (attrs, kwds) = state
+        self._cmemsubsets_parsed_ = attrs.pop('_cmemsubsets_parsed_')
+        self._set_all(**kwds)
+        self.__dict__.update(attrs)
+
+    def __getstate__(self):
+        attrs = dict(
+            (k, v) for (k, v) in self.__dict__.items()
+            if k not in ['_struct_', '_struct_p_', '_cdatastore_'])
+        kwds = {}
+        kwds.update(self._cdatastore_)
+        kwds.update(
+            (k, getattr(self, k)) for (k, v) in self._cmems_parsed_.items()
+            if v.valtype == 'scalar')
+        return (attrs, kwds)
+
+    def __copy__(self):
+        clone = self.__class__.__new__(self.__class__)
+        clone.__dict__.update(self.__dict__)
+        return clone
+
     def _is_cmem_scalar(self, name):
         return (name in self._cmems_parsed_ and
                 self._cmems_parsed_[name].valtype == 'scalar')
@@ -679,7 +708,9 @@ class SimObject(object):
                 dtype = CDT2DTYPE[parsed.cdt]
                 arr = numpy.zeros(shape, dtype=dtype)
                 self._cdatastore_[vname] = arr
-                if self._calloc_ and 1 < arr.ndim <= cstyle.MAXDIM:
+                if parsed.carrtype == "flat":
+                    ptr = arr.ctypes.data_as(POINTER(CDT2CTYPE[parsed.cdt]))
+                elif self._calloc_ and 1 < arr.ndim <= cstyle.MAXDIM:
                     cs = cstyle.CStyle(arr)
                     self._cdatastore_['CStyle:%s' % vname] = cs
                     ptr = cast(
